module KXML
    class SAXParser
        START_TAG           = 'START_TAG'
        END_TAG             = 'END_TAG'
        COMMENTS            = 'COMMENTS'
        SELF_CLOSE_TAG      = 'SELF_COLSE_TAG'
        DOCUMENT_STATEMENT  = 'DOCUMENT_STATEMENT'
        CONTENT             = 'CONTENT'
        
        attr_accessor :handler
        def initialize
            reset()
        end
        
        def reset
            @state = :NONE
            @data = ''
            @finish = false
        end
        
        def <<(data)
            push(data)
        end
        
        def push(data)
            @data += data
            parse()
        end
        
        def finish
            @finish = true
            parse()
        end
        
        private
        
        def parse
            raise "Cannot parse without handler" if @handler == nil
            return if @data == nil || @data == ''
            
            while(true)
                result = @data.match(/^<([\s\S.]*?)>/)
                if result !=nil && @data[0] == '<'
                    dispath_tag(result.to_s)
                    @data = @data[result[1].size+2,@data.size]
                    next
                end
                result = @data.match(/([\s\S.]+?)</)
                if result !=nil
                    dispath_content(result[1])
                    @data = @data[result[1].size,@data.size]
                    next
                end
                break
            end
            dispath_content(@data) if @finish && @data != nil && @data != ''
        end
        
        def dispath_tag(data)
            if data =~ /<\?xml\s+(.*?)\?>/
                param = parse_tag("STATEMENT #{data.match(/<\?xml\s+(.*?)\?>/)[1]}")
                param[:type] = DOCUMENT_STATEMENT
                @handler.call(param)
                return
            end
            if data =~ /<([\s\S.]*?)\/>/
                param = parse_tag(data.match(/<([\s\S.]*?)\/>/)[1])
                param[:type] = SELF_CLOSE_TAG
                @handler.call(param)
                return
            end
            if data =~ /<\/([\s\S.]*?)>/
                param = parse_tag(data.match(/<\/([\s\S.]*?)>/)[1])
                param[:type] = END_TAG
                @handler.call(param)
                return
            end
            if data =~ /<\!--([\s\S.]*?)-->/
                @handler.call({:type => COMMENTS , :name => data.match(/<\!--([\s\S.]*?)-->/)[1]})
                return
            end
            param = parse_tag(data.match(/<([\s\S.]*?)>/)[1])
            param[:type] = START_TAG
            @handler.call(param)
        end
        
        def dispath_content(data)
            @handler.call({:type => CONTENT , :name => data})
        end
        
        def parse_tag(data)
            data.strip!
            i = data.index(' ')
            if i == nil then
                return {:name => data}
            else
                ret = {}
                ret[:name] = data[0,i]
                ret[:attributes] = {}
                
                data.scan(/[a-zA-Z0-9\_\-]+=\"[\w\W]*?\"/){|s|
                    k = /[a-zA-Z0-9\_\-]+(?=(\b*=))/.match(s).to_s
                    di = s.index('"')
                    si = s.index("'")
                    if di == nil then
                        i = si
                    elsif si == nil then
                        i = di
                    else
                        i = si < di ? si : di
                    end
                    v = s[i+1,s.size-i-2]
                    ret[:attributes][k] = v
                }
                return ret
            end
        end
        
    end
    
end
